@@ -347,8 +347,8 @@ int CMP_CALLMODEL compareREAL(const void *current, const void *candidate)
    where interchanges are reflected in a caller-initialized integer "tags" list. */
 void hpsort(void *attributes, int count, int offset, int recsize, MYBOOL descending, findCompare_func findCompare)
 {
-  register int  i, j, k, ir, order;
-  register char *hold, *base;
+   int  i, j, k, ir, order;
+   char *hold, *base;
   char          *save;
 
   if(count < 2)
@@ -409,8 +409,8 @@ void hpsortex(void *attributes, int count, int offset, int recsize, MYBOOL desce
     return;
   }
   else {
-    register int  i, j, k, ir, order;
-    register char *hold, *base;
+     int  i, j, k, ir, order;
+     char *hold, *base;
     char          *save;
     int           savetag;
 
@@ -515,7 +515,7 @@ void QS_delete(UNIONTYPE QSORTrec a[], int ipos, int epos)
 }
 int QS_sort(UNIONTYPE QSORTrec a[], int l, int r, findCompare_func findCompare)
 {
-  register int i, j, nmove = 0;
+   int i, j, nmove = 0;
   UNIONTYPE QSORTrec v;
 
   /* Perform the a fast QuickSort */
